#!/usr/bin/env python3

# Open the entry point when program is executed as a module using 'python -m md2anki [...]'

# Local modules
from md2anki.entry_point import entry_point


if __name__ == "__main__":
    entry_point()
